#!/usr/bin/perl
# roqetpadcmd;
# Last updated: 2010-03-09
$version = '6.1';
$copyyear = '2010';

=head1 RoqetPadCmd v6

  RoqetPadCmd is a simple script to search and edit RoqetPad data on a 
  SQLite DB. For more functionality, use the web version of RoqetPad.

=head1 Author

  roqet (http://www.roqet.org/contact.html).

=head1 ChangeLog

  2010-01-02: Version 6.0 released. - Initial release for local testing.
  2010-03-09: Version 6.1 released. - Format modifications.

=head1 Copyright

  RoqetPadCmd copyright 2010 roqet - (http://www.roqet.org)
  RoqetPadCmd can be distributed and modified under the terms of the
  GNU General Public License: http://www.gnu.org/copyleft/gpl.html

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
  GNU General Public License for more details.

  Updates and other scripts and software available at:
  http://www.roqet.org/software.html
   ~~~
   'l'
    -

=cut

use DBI;

#
# read input parameters and/or environment variables
#
$database_instance = $ARGV[0];
if ($database_instance eq '') { die "error: no database instance file specified: $!\n"; }
$editor = $ARGV[1];

print "\nRoqetPad Command-line v$version Copyright $copyyear\n";
while ($command ne '.exit') {
  print "Enter .help for instructions\n";
  print "Command?: ";
  $input = <STDIN>;
  chomp($input);
  if ( $input eq '.exit' ) { last; }
  elsif ( substr($input,0,4) eq '.add' ) { 
      ($cmd,$txt) = split(' ',$input,2);
      dbadd ($txt);
  }
  elsif ( substr($input,0,5) eq '.edit' ) { 
      ($cmd,$txt) = split(' ',$input,2);
      dbedit ($txt);
  }
  elsif ( substr($input,0,7) eq '.delete' ) { 
      ($cmd,$txt) = split(' ',$input,2);
      dbdelete ($txt);
  }
  elsif ( substr($input,0,7) eq '.search' ) { 
      ($cmd,$txt) = split(' ',$input,2);
      dbsearch ($txt);
  }
  elsif ( substr($input,0,5) eq '.help' ) { 
      dbhelp ();
  }
  else { dbsearch ($input); }
}
unlink('editme.txt');
print "\nExit RoqetPad Command-line\n\n";

sub dbhelp {
#
# display instructions
#
  print ".exit - Exit RoqetPad Command-line\n";
  print ".add [alias] - Create a new RoqetPad entry\n";
  print ".edit [alias] - Edit an existing RoqetPad entry\n";
  print ".delete [alias] - Delete an existing RoqetPad entry\n";
  print ".search [alias] - Search for an existing RoqetPad entry\n";
  print ".help - Display this help\n";
}

sub dbadd {
#
# create an empty file to be edited
# then insert the entry into the database
#
  local($input) = @_;
  undef @collect;

  open(EDITFILE,"> editme.txt") or die "error opening editme.txt: $!\n";
  print EDITFILE "\n"; 
  close EDITFILE;
  system "$editor editme.txt";
  $editfile = '';    
  open EDITFILE, 'editme.txt' or die "error opening editme.txt: $!\n";
  while (<EDITFILE>) {
    $editfile = $editfile . $_;
  }
  close EDITFILE;
  dbconnect();
  $rc = $dbh->prepare( "INSERT INTO roqetpad (alias,detail) VALUES (?, ?)" );
  $rc->bind_param( 1, $input );
  $rc->bind_param( 2, $editfile );
  $rc->execute() or print "Error: $DBI::errstr\n";
  if ($rc ne '') { print "Entry Inserted\n"; }
  $rc->finish();
  dbdisconnect();
}

sub dbedit {
#
# run a select against the database and create a file to be edited
# then update the entry on the database
#
  local($input) = @_;
  undef @collect;

  dbconnect();

  $sth = $dbh->prepare("SELECT ALIAS,DETAIL FROM ROQETPAD WHERE LOWER(ALIAS) LIKE ?");
  $sth->bind_param( 1, "%$input%" );
  $sth->execute() or print "Error: collecting data!\n $DBI::errstr\n";
  $sth->bind_columns( undef, \$alias, \$detail );
  $aliascount = 0;
  while( $sth->fetch() ) {
    $editalias = $alias;
    #@details = split("\n",$detail);
    $details = $detail;
    $aliascount++;
  }
  $sth->finish();
  dbdisconnect();
  if (@details[0] eq '') { print "Selection returned No results!\n"; }
  elsif ($aliascount gt 1) { print "Selection returned More than 1 result; cannot edit!\n"; }
  else { 
    open(EDITFILE,"> editme.txt") or die "error opening editme.txt: $!\n";
    #print EDITFILE "@details"; 
    print EDITFILE "$details"; 
    close EDITFILE;
    system "$editor editme.txt";
    $editfile = '';    
    open EDITFILE, 'editme.txt' or die "error opening editme.txt: $!\n";
    while (<EDITFILE>) {
      $editfile = $editfile . $_;
    }
    close EDITFILE;
    dbconnect();
    $rc = $dbh->prepare( "UPDATE ROQETPAD SET DETAIL=? WHERE ALIAS=?" );
    $rc->bind_param( 1, $editfile );
    $rc->bind_param( 2, $editalias );
    $rc->execute() or print "Error: $DBI::errstr\n";
    if ($rc ne '') { print "Entry Updated\n"; }
    $rc->finish();
    dbdisconnect();
  }
}

sub dbdelete {
#
# delete an entry on the database
#
  local($input) = @_;
  undef @collect;

  dbconnect();

  $sth = $dbh->prepare("SELECT ALIAS,DETAIL FROM ROQETPAD WHERE LOWER(ALIAS) LIKE ?");
  $sth->bind_param( 1, "%$input%" );
  $sth->execute() or print "Error: collecting data!\n $DBI::errstr\n";
  $sth->bind_columns( undef, \$alias, \$detail );
  $aliascount = 0;
  while( $sth->fetch() ) {
    $editalias = $alias;
    @details = split("\n",$detail);
    $aliascount++;
  }
  $sth->finish();
  dbdisconnect();
  if (@details[0] eq '') { print "Selection returned No results!\n"; }
  elsif ($aliascount gt 1) { print "Selection returned More than 1 result; cannot delete multiple entries!\n"; }
  else { 
    dbconnect();
    $rc = $dbh->prepare( "DELETE FROM ROQETPAD WHERE ALIAS=?" );
    $rc->bind_param( 1, $input );
    $rc->execute() or print "Error: $DBI::errstr\n";
    if ($rc ne '') { print "Entry Deleted\n"; }
    $rc->finish();
    dbdisconnect();
  }
}

sub dbsearch {
#
# run a select against the database and print results
#
  local($input) = @_;
  undef @collect;

  dbconnect();

  $sth = $dbh->prepare("SELECT ALIAS,DETAIL FROM ROQETPAD WHERE LOWER(ALIAS) LIKE ?");
  $sth->bind_param( 1, "%$input%" );
  $sth->execute() or print "Error: collecting data!\n $DBI::errstr\n";
  $sth->bind_columns( undef, \$alias, \$detail );
  while( $sth->fetch() ) {
    push(@collect,"Alias: $alias\n"); 
    @details = split("\n",$detail);
    generatetext();
    push(@collect,"\n*******\n");
  }
  $sth->finish();
  dbdisconnect();
  if (@collect[0] eq '') { print "Selection returned No results!\n"; }
  else { print "\nResults:\n\n@collect\n"; }
}

sub generatetext {
#
# builds text from roqet document format text file
#
  local($source) = @_;
  undef (@body);
  $pretag = 0;
  foreach $line (@details) {
    chomp($line);
    if (substr($line,0,3) eq '-->') { 
      ($command,$filename) = split(/\:/,$line,2);
      open IMPORT, $filename or warn "import failure error opening " . $filename . ": $!\n";
      @importfile = <IMPORT>;    
      close IMPORT;
      push (@body, "\n");
      foreach $impline (@importfile) {
        chomp($impline);
        push (@body, $impline . "\n");
      }
      push (@body, "\n\n");
      undef (@importfile);
    }
    elsif (substr($line,0,2) eq '-=') { 
      $line =~ s/\-\=//g;
      $line =~ s/\=\-//g;
      push (@body, "* $line\n");
    }
    elsif (substr($line,0,1) eq '-') { next; }
    elsif ($line ne ''){
      if ($line =~ /\[\[/){ 
        $pretag = 1; 
        push (@body, "\n");
      }
      elsif ($line =~ /\]\]/){
        $pretag = 0; 
        push (@body, "\n");
      }
      elsif ( $pretag ){
        push (@body, $line  . "\n");
      }
      else {
        $line =~ s/mailto\://g;
        $line =~ s/\[//g;
        $line =~ s/\]//g;
        $line =~ s/\{\*//g;
        $line =~ s/\*\}//g;
        $line =~ s/\{\{//g;
        $line =~ s/\~\~/ \-\- /g;
        $line =~ s/\}\}//g;
        if (substr($line,0,2) eq '@ ') { 
          $line =~ s/\@ //g;
          $line = "** $line";
        }
        # new-style bullet
        $line =~ s/\*\*\* /* /g;
        $line =~ s/\*\-\*/\-\-\-/g;
        if ($line =~ /\= /){ 
          ($comm,$desc) = split(/\=/,$line,2);
          push (@body, "$comm: $desc\n");
        }
        else {
          push (@body, "$line\n");
        }
      }
    }	  
  }
  foreach $bit (@body) { push(@collect,"$bit"); }
}

sub dbconnect {
#
# connect to database
#
  $dbh = DBI->connect( "dbi:SQLite:$database_instance" )
    or print "Error: Can't connect to Sqlite database:\n $DBI::errstr!\nCheck your connection details.\n";
}

sub dbdisconnect {
#
# disconnect from database
#
  $dbh->disconnect 
    or print "Error: Database disconnect failed:\n $DBI::errstr!\n";
}


